<?php

namespace BigBite\Content_Query;

/**
 * Class responsible for finding blocks in content
 */
class Block_Parser {

	/**
	 * Recursively searches a multidimensional array of blocks
	 *
	 * @param string $target     Block Name to search for
	 * @param array  $attributes Attributes to match on each target block
	 * @param string $mode       A value representing the boolean operator to use when searching
	 * @param array  $blocks     Multidimensional array of blocks
	 *
	 * @return bool
	 */
	public static function has_block_with_attributes( string $target, array $attributes, string $mode, array $blocks ): bool {
		foreach ( $blocks as $block ) {
			if ( empty( $block['blockName'] ) ) {
				continue;
			}

			if (
				$block['blockName'] === $target &&
				static::attribute_checks( $attributes, $block['attrs'], $mode )
			) {
				return true;
			}

			if ( ! empty( $block['innerBlocks'] ) ) {
				if ( static::has_block_with_attributes( $target, $attributes, $mode, $block['innerBlocks'] ) ) {
					return true;
				};
			}
		}

		return false;
	}

	/**
	 * Match a blocks saved and default attribute values against
	 * a specific selection of attributes
	 *
	 * @param array  $attributes       Attributes to match on the target block
	 * @param array  $block_attributes The blocks saved/default attributes
	 * @param string $mode             Value representing the boolean operator to use when searching
	 *
	 * @return bool
	 */
	private static function attribute_checks( array $attributes, array $block_attributes, string $mode ): bool {
		$has_match = true;

		foreach ( $attributes as $attribute ) {
			// check for attribute on block and a default attribute value
			if (
				! array_key_exists( $attribute['name'], $block_attributes ) &&
				! isset( $attribute['defaultValue'] )
			) {
				// return early as running in 'AND' search mode
				if ( 'and' === $mode ) {
					return false;
				}

				// skip current attribute in 'OR' search mode
				$has_match = false;
				continue;
			}

			$attribute_type          = $attribute['type'];
			$payload_attribute_value = $attribute['value'];
			$block_attribute_value   = $block_attributes[ $attribute['name'] ] ?? $attribute['defaultValue'];
			$has_attribute_match     = Type_Comparison::compare( $attribute_type, $payload_attribute_value, $block_attribute_value );

			// short the attribute search loop as have found a 'true'
			if ( $has_attribute_match && 'or' === $mode ) {
				$has_match = true;
				break;
			}

			// sets match to false when searching blocks in OR mode and where the block had the attribute
			if ( ! $has_attribute_match && 'or' === $mode ) {
				$has_match = false;
				continue;
			}

			// short the attribute search loop as have found a 'false'
			if ( ! $has_attribute_match && 'and' === $mode ) {
				$has_match = false;
				break;
			}
		}

		return $has_match;
	}
}
