<?php

namespace BigBite\Content_Query;

/**
 * Class responsible for initialising plugin dashboard assets, menus and other
 */
final class Dashboard {

	/**
	 * Handle to be used when enqueuing styles/scripts
	 *
	 * @var string
	 */
	private const ENQUEUE_NAME = 'bigbite-content-query';

	/**
	 * Class constructor
	 */
	public function __construct() {
		add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_assets' ] );
		add_action( 'admin_menu', [ $this, 'add_menu' ] );
	}

	/**
	 * Conditionally enqueues plugin scripts and styles
	 *
	 * @param string $hook_suffix Current admin page
	 *
	 * @return void
	 */
	public function enqueue_assets( string $hook_suffix ): void {
		if ( 'toplevel_page_content-query' !== $hook_suffix ) {
			return;
		}

		$plugin_name = basename( BIGBITE_CONTENT_QUERY_PLUGIN_DIR );

		wp_enqueue_script(
			self::ENQUEUE_NAME,
			plugins_url( $plugin_name . '/dist/scripts/' . CONTENT_QUERY_DASHBOARD_JS ),
			[ 'wp-i18n', 'wp-url' ],
			CONTENT_QUERY_VERSION,
			true
		);

		wp_enqueue_style(
			self::ENQUEUE_NAME,
			plugins_url( $plugin_name . '/dist/styles/' . CONTENT_QUERY_DASHBOARD_CSS ),
			[],
			CONTENT_QUERY_VERSION
		);

		$dashboard_data = [
			'isMultisite'         => is_multisite(),
			'supportedPostTypes'  => Helpers::supported_post_types_with_label(),
			'supportedMediaTypes' => Helpers::supported_media_types(),
			'registeredBlocks'    => Helpers::get_registered_blocks(),
			'allowedSites'        => Helpers::allowed_sites(),
			'registeredPostMeta'  => array_values( Helpers::get_all_registered_post_meta( true ) ),
			'totalSites'          => is_multisite() ? count( get_sites() ) : 1,
			'restUrlBase'         => esc_url( get_rest_url() ),
			'restUrlPath'         => 'bigbite-content-query/v1',
			'restToken'           => wp_create_nonce( 'wp_rest' ),
			'version'             => CONTENT_QUERY_VERSION,
			'plugins'             => Helpers::get_plugins(),
			'themes'              => is_multisite() ? Helpers::get_themes() : [],
			'isDebugMode'         => defined( 'WP_DEBUG' ) && WP_DEBUG === true,
			'pageRequestMaximum'  => Helpers::page_request_maximum(),
		];

		wp_add_inline_script(
			self::ENQUEUE_NAME,
			sprintf( 'const bigbiteContentQueryData = Object.freeze(%s);', wp_json_encode( $dashboard_data ) ),
			'before'
		);
	}

	/**
	 * Adds plugin navigation menu to Dashboard sidebar
	 *
	 * @return void
	 */
	public function add_menu(): void {
		$sub_menu_pages = array_filter( [
			[
				'title' => __( 'Dashboard', 'bigbite-content-query' ),
				'slug'  => 'dashboard',
			],
			[
				'title' => __( 'Post Explorer', 'bigbite-content-query' ),
				'slug'  => 'post-explorer',
			],
			[
				'title'          => __( 'Plugin Locator', 'bigbite-content-query' ),
				'slug'           => 'plugin-locator',
				'multisite_only' => true,
			],
			[
				'title'          => __( 'Theme Locator', 'bigbite-content-query' ),
				'slug'           => 'theme-locator',
				'multisite_only' => true,
			],
			[
				'title' => __( 'Media Explorer', 'bigbite-content-query' ),
				'slug'  => 'media-explorer',
			],
			[
				'title' => __( 'Post Meta Search', 'bigbite-content-query' ),
				'slug'  => 'post-meta-search',
			],
			[
				'title' => __( 'Block Search', 'bigbite-content-query' ),
				'slug'  => 'block-search',
			],
		], fn( $view ) => ! ( ! is_multisite() && isset( $view['multisite_only'] ) ) );

		$menu_icon = false;
		if ( is_readable( BIGBITE_CONTENT_QUERY_PLUGIN_DIR . '/dist/static/images/cq.svg' ) ) {
			$menu_icon = sprintf(
				'data:image/svg+xml;base64,%s',
				base64_encode( file_get_contents( BIGBITE_CONTENT_QUERY_PLUGIN_DIR . '/dist/static/images/cq.svg' ) )
			);
		}

		add_menu_page(
			__( 'Content Query', 'bigbite-content-query' ),
			__( 'Content Query', 'bigbite-content-query' ),
			Helpers::supported_user_capability(),
			'content-query',
			[ $this, 'display_dashboard' ],
			$menu_icon,
		);

		foreach ( $sub_menu_pages as $sub_menu_page ) {
			add_submenu_page(
				'content-query',
				$sub_menu_page['title'],
				$sub_menu_page['title'],
				Helpers::supported_user_capability(),
				'content-query#' . $sub_menu_page['slug'],
				[ $this, 'display_dashboard' ]
			);
		}
	}

	/**
	 * Callback function for content to be used in menu inits
	 *
	 * @return void
	 */
	public function display_dashboard(): void {
		require_once realpath( BIGBITE_CONTENT_QUERY_PLUGIN_DIR . '/inc/views/dashboard.php' );
	}
}
