<?php

namespace BigBite\Content_Query;

use WP_Block_Type;
use WP_Block_Type_Registry;

/**
 * A collection of helper functions
 */
final class Helpers {

	/**
	 * Returns a list of post types which support blocks
	 *
	 * @return array
	 */
	public static function supported_post_types(): array {
		$supported_post_types = [ 'post', 'page', 'wp_block', 'wp_template', 'wp_template_part' ];

		/**
		 * Filters supported post types array
		 *
		 * @param array $supported_post_types An array of supported post types
		 */
		$supported_post_types = apply_filters( 'bigbite_content_query_supported_post_types', $supported_post_types );

		return array_values( array_filter( $supported_post_types, 'post_type_exists' ) );
	}

	/**
	 * Returns a list of supported post statuses
	 *
	 * @return array
	 */
	public static function supported_post_statuses(): array {
		$supported_post_statuses  = [ 'publish', 'draft', 'future', 'pending' ];
		$registered_post_statuses = array_keys( get_post_stati() );

		/**
		 * Filters supported post statuses array
		 *
		 * @param array $supported_post_statuses An array of supported post statuses
		 */
		$supported_post_statuses = apply_filters( 'bigbite_content_query_supported_post_statuses', $supported_post_statuses );

		return array_intersect( $supported_post_statuses, $registered_post_statuses );
	}

	/**
	 * Returns total post type count from supported post statuses
	 * 
	 * @param string $post_type Post type
	 *
	 * @return integer
	 */
	public static function post_type_post_status_total_count( string $post_type = 'post' ): int {
		$statuses = self::supported_post_statuses();
		$totals   = wp_count_posts( $post_type );
		$total    = 0;

		foreach ( $statuses as $status ) {
			$total += $totals?->$status ?? 0;
		}

		return $total;
	}

	/**
	 * Returns the capability a user must have for dashboard functionality
	 *
	 * @return string
	 */
	public static function supported_user_capability(): string {
		$supported_user_capability = 'remove_users';

		/**
		 * Filters the required user capability for plugin functionality
		 *
		 * @param string $supported_user_capability Required user capability
		 */
		$supported_user_capability = apply_filters( 'bigbite_content_query_supported_user_capability', $supported_user_capability );

		return $supported_user_capability;
	}

	/**
	 * Returns a list of post types which support blocks with their label
	 *
	 * @return array
	 */
	public static function supported_post_types_with_label(): array {
		$post_types            = self::supported_post_types();
		$post_types_with_label = [];
		foreach ( $post_types as $post_type ) {
			$label                   = get_post_type_object( $post_type )?->labels?->name;
			$post_types_with_label[] = [
				'key'   => $post_type,
				'label' => $label ?? $post_type,
				'count' => wp_count_posts( $post_type )->publish,
			];
		}

		return $post_types_with_label;
	}

	/**
	 * Returns a list of supported media types
	 *
	 * @return array
	 */
	public static function supported_media_types(): array {
		$supported_media_types = [ 'image', 'video', 'application', 'audio' ];

		/**
		 * Filters supported media types array
		 *
		 * @param array $supported_media_types An array of supported media types
		 */
		$supported_media_types = apply_filters( 'bigbite_content_query_supported_media_types', $supported_media_types );

		return $supported_media_types;
	}

	/**
	 * Returns a list of registered blocks with filtered properties
	 *
	 * @return array
	 */
	public static function get_registered_blocks(): array {
		$registered_blocks          = WP_Block_Type_Registry::get_instance()->get_all_registered();
		$registered_blocks_filtered = [];
		$unwanted_properties        = [
			'styles',
			'variations',
			'supports',
			'example',
			'render_callback',
			'uses_context',
			'provides_context',
			'editor_script',
			'script',
			'view_script',
			'editor_style',
			'style',
			'ancestor',
			'editor_script_handles',
			'editor_style_handles',
			'icon',
			'parent',
			'script_handles',
			'style_handles',
			'textdomain',
			'view_script_handles',
			'keywords',
			'api_version',
		];

		$readable_properties = get_class_vars( WP_Block_Type::class );

		foreach ( $registered_blocks as $registered_block ) {
			foreach ( $unwanted_properties as $unwanted_property ) {
				if ( in_array( $unwanted_property, $readable_properties, true ) ) {
					unset( $registered_block->$unwanted_property );
				}
			}

			$temp_attributes = [];
			foreach ( $registered_block->attributes as $key => $value ) {
				$value['name']     = $key;
				$temp_attributes[] = $value;
			}

			$registered_block->attributes = $temp_attributes;
			$registered_blocks_filtered[] = $registered_block;
		}

		/**
		 * Filters the list of registered blocks
		 *
		 * @param array $registered_blocks_filtered A list of registered blocks
		 */
		return apply_filters( 'bigbite_content_query_registered_blocks', $registered_blocks_filtered );
	}

	/**
	 * Returns an array of data from the current site
	 *
	 * @return array
	 */
	private static function allowed_single_site_data(): array {
		if ( ! current_user_can( self::supported_user_capability() ) ) {
			return [];
		}

		return [
			[
				'id'         => get_current_blog_id(),
				'site_title' => empty( get_bloginfo( 'name' ) ) ? __( '(no title)' ) : get_bloginfo( 'name' ),
				'dashboard'  => get_dashboard_url(),
				'default'    => true,
			],
		];
	}

	/**
	 * Returns an array of data from all active sites in a multisite network
	 *
	 * @return array
	 */
	private static function allowed_multisite_super_admin_site_data(): array {
		$args          = [
			'archived' => 0,
			'deleted'  => 0,
			'spam'     => 0,
		];
		$all_sites     = get_sites( $args );
		$allowed_sites = [];

		if ( ! is_array( $all_sites ) || count( $all_sites ) < 1 ) {
			return [];
		}

		foreach ( $all_sites as $site ) {
			if ( empty( $site->blog_id ) ) {
				continue;
			}

			$allowed_sites[] = [
				'id'         => $site->blog_id,
				'site_title' => empty( get_blog_option( $site->blog_id, 'blogname' ) ) ? __( '(no title)' ) : get_blog_option( $site->blog_id, 'blogname' ),
				'dashboard'  => get_admin_url( $site->blog_id ),
				'default'    => get_current_blog_id() === (int) $site->blog_id,
			];
		}

		return $allowed_sites;
	}

	/**
	 * Returns an array of data from sites in a multisite network
	 * where the current user has the required capability
	 *
	 * @return array
	 */
	private static function allowed_multisite_capability_site_data(): array {
		$allowed_sites   = [];
		$current_user_id = get_current_user_id();
		$users_sites     = get_blogs_of_user( $current_user_id );

		if ( ! is_array( $users_sites ) || count( $users_sites ) < 1 ) {
			return [];
		}

		foreach ( $users_sites as $site ) {
			if ( empty( $site->userblog_id ) ) {
				continue;
			}

			$is_user_allowed = current_user_can_for_blog( $site->userblog_id, self::supported_user_capability() );

			if ( ! $is_user_allowed ) {
				continue;
			}

			$allowed_sites[] = [
				'id'         => $site->userblog_id,
				'site_title' => empty( get_blog_option( $site->userblog_id, 'blogname' ) ) ? __( '(no title)' ) : get_blog_option( $site->userblog_id, 'blogname' ),
				'dashboard'  => get_admin_url( $site->userblog_id ),
				'default'    => get_current_blog_id() === $site->userblog_id,
			];
		}

		return $allowed_sites;
	}

	/**
	 * Returns an array of sites that a user is allowed to query in any multisite
	 * network queries
	 *
	 * @return array
	 */
	public static function allowed_sites(): array {
		if ( is_multisite() ) {
			if ( is_super_admin() ) {
				return self::allowed_multisite_super_admin_site_data();
			}

			return self::allowed_multisite_capability_site_data();
		}

		return self::allowed_single_site_data();
	}

	/**
	 * Returns a list of installed themes
	 *
	 * @return array
	 */
	public static function get_themes() {
		$themes          = wp_get_themes();
		$themes_detailed = [];

		foreach ( $themes as $theme ) {
			$themes_detailed[] = [
				'name'      => esc_html( $theme ),
				'theme_uri' => esc_html( $theme->get( 'ThemeURI' ) ),
				'author'    => esc_html( $theme->get( 'Author' ) ),
				'version'   => esc_html( $theme->get( 'Version' ) ),
				'parent'    => esc_html( $theme->get( 'Template' ) ),
			];
		}

		return $themes_detailed;
	}

	/**
	 * Returns a list of plugins and active statuses
	 *
	 * @return array
	 */
	public static function get_plugins(): array {
		$plugins     = get_plugins();
		$plugin_keys = array_keys( $plugins );

		if ( 0 === count( $plugins ) ) {
			return [];
		}

		foreach ( $plugin_keys as $plugin ) {
			if ( is_multisite() && is_plugin_active_for_network( $plugin ) ) {
				$plugins[ $plugin ]['network_active'] = true;
				continue;
			}

			if ( is_plugin_active( $plugin ) ) {
				$plugins[ $plugin ]['active'] = true;
			}
		}

		return $plugins;
	}

	/**
	 * Returns an array of all registered post meta across
	 * the supported post types
	 * 
	 * @param boolean $is_flat Mode of operation
	 *
	 * @return array
	 */
	public static function get_all_registered_post_meta( $is_flat = false ): array {
		$post_meta = [];

		// base object post meta
		$base_post_meta = get_registered_meta_keys( 'post' );

		if ( ! empty( $base_post_meta ) ) {
			$post_meta['base'] = $base_post_meta;
		}

		// object subtype post meta
		foreach ( self::supported_post_types() as $post_type ) {
			$post_type_post_meta = get_registered_meta_keys( 'post', $post_type );

			if ( empty( $post_type_post_meta ) ) {
				continue;
			}

			$post_meta[ $post_type ] = $post_type_post_meta;
		}

		/**
		 * Filters the list of registered post meta
		 *
		 * @param array $post_meta A list of registered post meta
		 */
		$post_meta = apply_filters( 'bigbite_content_query_supported_post_meta', $post_meta );

		if ( $is_flat ) {
			$post_meta = self::flattened_registered_post_meta( $post_meta );
		}

		return array_unique( $post_meta, SORT_REGULAR );
	}

	/**
	 * Returns an array of all registered post meta across
	 * the supported post types in a flattened array
	 * 
	 * @param array $post_meta Registered post meta
	 *
	 * @return array
	 */
	private static function flattened_registered_post_meta( array $post_meta ): array {
		$flattened_post_meta = [];

		foreach ( $post_meta as $metas ) {
			foreach ( $metas as $key => $value ) {
				// handle meta with same type and key but with different default values and target post type
				$current_keys    = array_column( $flattened_post_meta, 'key' );
				$found_key_index = array_search( $key, $current_keys, true );

				// make sure key and type both match
				if ( false !== $found_key_index && $flattened_post_meta[ $found_key_index ]['type'] === $value['type'] ) {
					if ( ! isset( $value['default'] ) ) { 
						continue;
					}

					// flag the entry that it has multiple defaults and group all defaults into one
					$flattened_post_meta[ $found_key_index ]['hasMultipleDefaults'] = true;
					$flattened_post_meta[ $found_key_index ]['default']             = array_unique(
						array_merge(
							[ $flattened_post_meta[ $found_key_index ]['default'] ],
							[ $value['default'] ]
						),
						SORT_REGULAR
					);

					continue;
				}

				$data = [
					'key'         => $key,
					'type'        => $value['type'],
					'description' => $value['description'] ?? '',
				];

				if ( isset( $value['default'] ) ) {
					$data['default'] = $value['default'];
				}

				$flattened_post_meta[] = $data;
			}
		}

		return $flattened_post_meta;
	}

	/**
	 * Maximum number of page requests allowed when requests are
	 * split into batches of pages per post type
	 * 
	 * @return integer
	 */
	public static function page_request_maximum(): int {
		/**
		 * Filters the maximum number of page requests allowed per batched search
		 *
		 * @param integer Page request maximum
		 */
		$maximum_pages = apply_filters( 'bigbite_content_query_page_request_maximum', 20 );

		return $maximum_pages;
	}
}
