<?php

namespace BigBite\Content_Query;

/**
 * Class responsible for finding post meta in posts
 */
final class Post_Meta_Parser {

	/**
	 * Checks a post id for a post meta match from user defined query
	 *
	 * @param array   $post_meta                Post meta to match on the target post id
	 * @param string  $mode                     Value representing the boolean operator to use when searching
	 * @param integer $post_id                  Current post id
	 * @param array   $all_registered_post_meta All registered post meta on current site
	 *
	 * @return bool
	 */
	public static function check_post_meta( array $post_meta, string $mode, int $post_id, array $all_registered_post_meta ): bool {
		$has_match              = true;
		$current_post_post_meta = get_post_meta( $post_id );

		foreach ( $post_meta as $meta ) {
			$current_post_meta_default = self::find_registered_meta_default( $all_registered_post_meta, $meta, $post_id );

			if (
				! array_key_exists( $meta['key'], $current_post_post_meta ) &&
				is_null( $current_post_meta_default )
			) {
				// return early as running in AND search mode
				if ( 'and' === $mode ) {
					return false;
				}

				// skip current meta key in OR search mode
				$has_match = false;
				continue;
			}

			$meta_type                    = $meta['type'];
			$meta_value                   = $meta['value'];
			$current_page_post_meta_value = $current_post_post_meta[ $meta['key'] ][0] ?? $current_post_meta_default;

			$has_post_meta_match = Type_Comparison::compare( $meta_type, $meta_value, $current_page_post_meta_value );

			// short the search loop as have found a match and in OR mode
			if ( $has_post_meta_match && 'or' === $mode ) {
				$has_match = true;
				break;
			}

			// sets match to false when searching post meta in OR mode and where the post meta was set but no match
			if ( ! $has_post_meta_match && 'or' === $mode ) {
				$has_match = false;
				continue;
			}

			// short the search loop as we didn't match the value and in AND mode
			if ( ! $has_post_meta_match && 'and' === $mode ) {
				$has_match = false;
				break;
			}
		}

		return $has_match;
	}

	/**
	 * Finds a default value for a meta key if there is one set when post meta is registered
	 *
	 * @param array   $all_registered_post_meta All registered post meta on current site
	 * @param array   $meta                     Meta object
	 * @param integer $post_id                  Current post id
	 *
	 * @return mixed
	 */
	private static function find_registered_meta_default( array $all_registered_post_meta, array $meta, int $post_id ): mixed {
		$current_post_meta_default = $all_registered_post_meta['base'][ $meta['key'] ]['default'] ?? null;

		if ( ! is_null( $current_post_meta_default ) ) {
			return $current_post_meta_default;
		}

		$current_post_post_type = get_post_type( $post_id );

		return $all_registered_post_meta[ $current_post_post_type ][ $meta['key'] ]['default'] ?? null;
	}
}
