<?php

namespace BigBite\Content_Query;

/**
 * Class responsible for type comparisons
 */
final class Type_Comparison {

	/**
	 * Checks values against specific rules for the type
	 *
	 * @param string $type   Type of comparison to use
	 * @param mixed  $value1 Value to be compared
	 * @param mixed  $value2 Value to be compared
	 *
	 * @return bool
	 */
	public static function compare( $type, $value1, $value2 ): bool {
		switch ( $type ) {
			case 'boolean':
				return static::boolean_comparison( $value1, $value2 );
			case 'object':
				return static::object_comparison( $value1, $value2 );
			case 'array':
				return static::array_comparison( $value1, $value2 );
			case 'integer':
				return static::integer_comparison( $value1, $value2 );
			case 'number':
				return static::number_comparison( $value1, $value2 );
			case 'string':
			default:
				return static::string_comparison( $value1, $value2 );
		}

		return false;
	}

	/**
	 * Boolean type value comparison checks
	 *
	 * @param mixed $value1 Value to be compared
	 * @param mixed $value2 Value to be compared
	 *
	 * @return bool
	 */
	private static function boolean_comparison( $value1, $value2 ): bool {
		$value1 = filter_var( $value1, FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE );
		$value2 = filter_var( $value2, FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE );

		if ( is_null( $value1 ) || is_null( $value2 ) ) {
			return false;
		}

		return (bool) $value1 === (bool) $value2;
	}

	/**
	 * Object type value comparison checks
	 *
	 * @param mixed $value1 Value to be compared
	 * @param mixed $value2 Value to be compared
	 *
	 * @return bool
	 */
	private static function object_comparison( $value1, $value2 ): bool {
		return static::flat_array_object_basic_string_comparison( $value1, $value2 );
	}

	/**
	 * Array type value comparison checks
	 *
	 * @param mixed $value1 Value to be compared
	 * @param mixed $value2 Value to be compared
	 *
	 * @return bool
	 */
	private static function array_comparison( $value1, $value2 ): bool {
		return static::flat_array_object_basic_string_comparison( $value1, $value2 );
	}

	/**
	 * String type value comparison checks
	 *
	 * @param mixed $value1 Value to be compared
	 * @param mixed $value2 Value to be compared
	 *
	 * @return bool
	 */
	private static function string_comparison( $value1, $value2 ): bool {
		if ( ! is_scalar( $value1 ) || ! is_scalar( $value2 ) ) {
			return false;
		}

		return (string) $value1 === (string) $value2;
	}

	/**
	 * Integer type value comparison checks
	 *
	 * @param mixed $value1 Value to be compared
	 * @param mixed $value2 Value to be compared
	 *
	 * @return bool
	 */
	private static function integer_comparison( $value1, $value2 ): bool {
		if ( ! ctype_digit( (string) $value1 ) || ! ctype_digit( (string) $value2 ) ) {
			return false;
		}

		return (int) $value1 === (int) $value2;
	}

	/**
	 * Number type value comparison checks
	 *
	 * @param mixed $value1 Value to be compared
	 * @param mixed $value2 Value to be compared
	 *
	 * @return bool
	 */
	private static function number_comparison( $value1, $value2 ): bool {
		return static::integer_comparison( $value1, $value2 );
	}

	/**
	 * Basic string comparison of two json encoded arrays/objects
	 *
	 * @param string $value1 A json string
	 * @param mixed  $value2 A data object/array
	 *
	 * @return bool
	 */
	private static function flat_array_object_basic_string_comparison( string $value1, $value2 ): bool {
		$value1 = json_decode( $value1, true );

		if ( is_null( $value1 ) || ! is_array( $value2 ) ) {
			return false;
		}

		ksort( $value1 );
		ksort( $value2 );

		$value1_string = wp_json_encode( $value1 );
		$value2_string = wp_json_encode( $value2 );

		return $value1_string === $value2_string;
	}

}
