<?php

namespace BigBite\Content_Query;

/**
 * Abstract class which all rest api classes should be extended from
 */
abstract class Rest_API_Base {

	/**
	 * REST API namespace
	 *
	 * @var string
	 */
	protected string $namespace = 'bigbite-content-query/v1';

	/**
	 * A posts_per_page value to be used and set in any extended classes
	 *
	 * @var int
	 */
	protected int $posts_per_page;

	/**
	 * An array of supported post types
	 *
	 * @var array
	 */
	protected array $supported_post_types = [];

	/**
	 * Class constructor
	 */
	public function __construct() {
		$this->supported_post_types = Helpers::supported_post_types();
	}

	/**
	 * Permission callback for allowing access to the rest api endpoints
	 *
	 * @return bool
	 */
	public function rest_api_permissions_check(): bool {
		return is_user_logged_in() && current_user_can( Helpers::supported_user_capability() );
	}

	/**
	 * Network site permission callback
	 *
	 * Check to make sure that the current request which involves a multisite network
	 * query has originated from a user who has the required capability at that
	 * site on the network
	 *
	 * A generic invalid parameter response is returned for security reasons
	 *
	 * @param mixed $site_id the site id on the multisite network
	 *
	 * @return bool
	 */
	public function network_site_permission_check( mixed $site_id ): bool {
		if ( ! is_multisite() ) {
			return true;
		}

		$site_id = absint( $site_id );

		return is_user_logged_in() && get_site( $site_id ) && current_user_can_for_blog( $site_id, Helpers::supported_user_capability() );
	}

	/**
	 * Validation callback for the 'post_type' request parameter
	 *
	 * @param mixed $param Current request parameter value
	 *
	 * @return bool
	 */
	public function post_type_validate_callback( mixed $param ): bool {
		return in_array( $param, $this->supported_post_types, true );
	}

	/**
	 * Abstract function that all extensions of this class must implement
	 * to register their rest routes
	 *
	 * @return void
	 */
	abstract public function register_routes(): void;
}
