<?php

namespace BigBite\Content_Query;

use WP_Post;
use WP_Error;
use WP_REST_Request;
use WP_REST_Response;

/**
 * Class responsible for registering the block-search endpoint
 */
class Rest_API_Block_Search extends Rest_API_Base {

	/**
	 * Class constructor
	 */
	public function __construct() {
		parent::__construct();

		/**
		 * Filters the posts per page parameter when searching for blocks
		 *
		 * @param integer Posts per page
		 */
		$this->posts_per_page = apply_filters( 'bigbite_content_query_block_search_posts_per_page_query', 5000 );
	}

	/**
	 * Registers the rest api endpoint for this class
	 *
	 * @return void
	 */
	public function register_routes(): void {
		register_rest_route(
			$this->namespace, '/block-search/', [
				'methods'             => 'POST',
				'callback'            => [ $this, 'block_search' ],
				'permission_callback' => [ $this, 'rest_api_permissions_check' ],
				'args'                => [
					'post_type'  => [
						'type'              => 'string',
						'required'          => true,
						'validate_callback' => [ $this, 'post_type_validate_callback' ],
						'sanitize_callback' => 'sanitize_text_field',
					],
					'page'       => [
						'type'              => 'integer',
						'required'          => true,
						'validate_callback' => [ $this, 'page_validate_callback' ],
						'sanitize_callback' => 'absint',
					],
					'block_name' => [
						'type'              => 'string',
						'required'          => true,
						'validate_callback' => [ $this, 'block_name_validate_callback' ],
						'sanitize_callback' => 'sanitize_text_field',
					],
					'mode'       => [
						'type'              => 'string',
						'required'          => false,
						'validate_callback' => [ $this, 'mode_validate_callback' ],
						'sanitize_callback' => 'sanitize_text_field',
					],
					'site_id'    => [
						'type'              => 'integer',
						'required'          => true,
						'default'           => get_current_blog_id(),
						'validate_callback' => [ $this, 'network_site_permission_check' ],
						'sanitize_callback' => 'absint',
					],
				],
			]
		);
	}

	/**
	 * Validation callback for the 'page' request parameter
	 *
	 * Makes sure that the 'page' request parameter is not larger than the total pages
	 *
	 * @param mixed           $param   Current request parameter value
	 * @param WP_REST_Request $request Request data
	 *
	 * @return bool
	 */
	public function page_validate_callback( mixed $param, WP_REST_Request $request ): bool {
		if ( ! is_int( $param ) && ! ctype_digit( (string) $param ) ) {
			return false;
		}

		$requested_post_type       = $request->get_param( 'post_type' );
		$site_id                   = $request->get_param( 'site_id' );
		$is_requested_current_site = get_current_blog_id() === $site_id;

		if ( ! $this->post_type_validate_callback( $requested_post_type ) ) {
			return false;
		}
		
		if ( is_multisite() && ! $is_requested_current_site ) {
			if ( ! $this->network_site_permission_check( $site_id ) ) {
				return false;
			}

			switch_to_blog( $site_id );
		}

		$post_type_count       = Helpers::post_type_post_status_total_count( $requested_post_type );
		$post_type_total_pages = ceil( $post_type_count / $this->posts_per_page );

		if ( is_multisite() && ! $is_requested_current_site ) {
			restore_current_blog();
		}

		if ( (int) $param <= (int) $post_type_total_pages ) {
			return true;
		}

		return false;
	}

	/**
	 * Validation callback for the 'block_name' request parameter in block_search
	 *
	 * @param mixed $param Current request parameter value
	 *
	 * @return bool
	 */
	public function block_name_validate_callback( mixed $param ): bool {
		return ! empty( $param ) && is_string( $param );
	}

	/**
	 * Validation callback for the 'mode' request parameter in block_search
	 *
	 * @param mixed $param Current request parameter value
	 *
	 * @return bool
	 */
	public function mode_validate_callback( mixed $param ): bool {
		return ! empty( $param ) && in_array( strtolower( $param ), [ 'and', 'or' ], true );
	}

	/**
	 * Search for blocks and attributes across a single site or multiple in
	 * a multisite installation
	 *
	 * @param WP_REST_Request $request Request data
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	public function block_search( WP_REST_Request $request ): WP_REST_Response | WP_Error {
		$json_body    = $request->get_json_params();
		$block_name   = $request->get_param( 'block_name' );
		$attributes   = $json_body['attributes'] ?? [];
		$mode         = $request->get_param( 'mode' ) ?? 'and';
		$post_type    = $request->get_param( 'post_type' );
		$page         = $request->get_param( 'page' );
		$site_id      = $request->get_param( 'site_id' );
		$is_multisite = is_multisite();
		$found_items  = [];

		if ( $is_multisite ) {
			switch_to_blog( $site_id );
		}

		$post_type_count       = Helpers::post_type_post_status_total_count( $post_type );
		$post_type_total_pages = ceil( $post_type_count / $this->posts_per_page );

		$args = [
			'post_type'      => $post_type,
			'posts_per_page' => $this->posts_per_page,
			'paged'          => $page,
			'post_status'    => Helpers::supported_post_statuses(),
		];

		$posts = get_posts( $args );

		foreach ( $posts as $post ) {
			$processed = $this->process_post( $block_name, $post, $attributes, $mode, $site_id );

			if ( is_null( $processed ) ) {
				continue;
			}

			$found_items[] = $processed;

			unset( $processed );
		}

		if ( $is_multisite ) {
			restore_current_blog();
		}

		$response = rest_ensure_response( $found_items );
		$response->header( 'X-WP-TotalPages', $post_type_total_pages );
		return $response;
	}

	/**
	 * Check if a post should be included in the result set
	 *
	 * @param string  $block_name The block being searched for
	 * @param WP_Post $post       The post object
	 * @param array   $attributes An array of attributes on the block to search against
	 * @param string  $mode       Search mode operator
	 * @param integer $site_id    Site id in a multisite network or single install
	 *
	 * @return array|null
	 */
	private function process_post( string $block_name, WP_Post $post, array $attributes, string $mode, int $site_id ): ?array {
		if ( ! has_block( $block_name, $post->ID ) ) {
			return null;
		}

		if ( count( $attributes ) > 0 ) {
			$parsed_blocks             = parse_blocks( $post->post_content );
			$has_block_with_attributes = Block_Parser::has_block_with_attributes( $block_name, $attributes, $mode, $parsed_blocks );

			if ( ! $has_block_with_attributes ) {
				return null;
			}
		}

		return [
			'id'          => $post->ID,
			'title'       => $post->post_title,
			'slug'        => $post->post_name,
			'post_type'   => $post->post_type,
			'guid'        => $post->guid,
			'post_date'   => $post->post_date,
			'edit'        => get_edit_post_link( $post->ID, null ),
			'site_title'  => get_bloginfo( 'name' ),
			'site_id'     => $site_id,
			'post_url'    => is_post_publicly_viewable( $post->ID ) ? get_permalink( $post->ID ) : '',
			'post_status' => $post->post_status,
		];
	}
}
