<?php

namespace BigBite\Content_Query;

use WP_Error;
use WP_REST_Request;
use WP_REST_Response;

/**
 * Class responsible for registering the media-search endpoint
 */
class Rest_API_Media_Search extends Rest_API_Base {

	/**
	 * Post meta keys to be included in the search
	 *
	 * @var array
	 */
	private array $post_meta_keys;

	/**
	 * Class constructor
	 */
	public function __construct() {
		parent::__construct();

		/**
		 * Filters the posts per page integer used when searching for media
		 *
		 * @param integer Posts per page value to be used in query
		 */
		$this->posts_per_page = apply_filters( 'bigbite_content_query_media_search_posts_per_page_query', 2500 );

		/**
		 * Filters the post meta keys array used when searching for media
		 *
		 * @param array A list of post meta keys to be checked when searching
		 */
		$this->post_meta_keys = apply_filters( 'bigbite_content_query_media_search_post_meta_keys', [] );
	}

	/**
	 * Registers the rest api endpoint for this class
	 *
	 * @return void
	 */
	public function register_routes(): void {
		register_rest_route(
			$this->namespace, '/media-search/', [
				'methods'             => 'GET',
				'callback'            => [ $this, 'media_search' ],
				'permission_callback' => [ $this, 'rest_api_permissions_check' ],
				'args'                => [
					'id'        => [
						'type'     => 'integer',
						'required' => true,
					],
					'post_type' => [
						'type'              => 'string',
						'required'          => true,
						'validate_callback' => [ $this, 'post_type_validate_callback' ],
						'sanitize_callback' => 'sanitize_text_field',
					],
					'page'      => [
						'type'              => 'integer',
						'required'          => true,
						'validate_callback' => [ $this, 'page_validate_callback' ],
						'sanitize_callback' => 'absint',
					],
				],
			]
		);
	}

	/**
	 * Validation callback for the 'page' request parameter in media_finder
	 *
	 * Makes sure that the 'page' request parameter is not larger than the total pages
	 * for the requested post type
	 *
	 * @param mixed           $param   Current request parameter value
	 * @param WP_REST_Request $request Request data
	 *
	 * @return bool
	 */
	public function page_validate_callback( mixed $param, WP_REST_Request $request ): bool {
		if ( ! is_int( $param ) && ! ctype_digit( (string) $param ) ) {
			return false;
		}

		$requested_post_type = $request->get_param( 'post_type' );

		if ( ! $this->post_type_validate_callback( $requested_post_type ) ) {
			return false;
		}

		$post_type_count       = Helpers::post_type_post_status_total_count( $requested_post_type );
		$post_type_total_pages = ceil( $post_type_count / $this->posts_per_page );

		if ( (int) $param <= (int) $post_type_total_pages ) {
			return true;
		}

		return false;
	}

	/**
	 * Search across a post types for posts using the requested media/attachment id
	 *
	 * @param WP_REST_Request $request Request data
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	public function media_search( WP_REST_Request $request ): WP_REST_Response | WP_Error {
		$media_id   = $request->get_param( 'id' );
		$post_type  = $request->get_param( 'post_type' );
		$page       = $request->get_param( 'page' );
		$media_meta = wp_get_attachment_metadata( $media_id );

		if ( ! $media_meta ) {
			return new WP_Error(
				'attachment_error',
				'Error finding attachment',
				[
					'status' => 404,
				]
			);
		}

		$is_image    = wp_attachment_is( 'image', $media_id );
		$found_items = [];
		$media_urls  = [];

		if ( $is_image ) {
			$media_urls = $this->generate_image_media_urls( $media_meta, $media_id );
		} else {
			$media_urls = $this->generate_other_media_urls( $media_id );
		}

		if ( is_wp_error( $media_urls ) ) {
			return $media_urls;
		}

		$post_type_count       = Helpers::post_type_post_status_total_count( $post_type );
		$post_type_total_pages = ceil( $post_type_count / $this->posts_per_page );

		$post_pages = array_chunk( $this->get_page( $post_type, $page ), 10 );

		foreach ( $post_pages as $post_page ) {
			$found_items = array_merge( $found_items, $this->process_page( $post_page, $is_image, $media_urls, $media_id ) );
		}

		unset( $post_pages, $post_page );

		$response = rest_ensure_response( $found_items );
		$response->header( 'X-WP-TotalPages', $post_type_total_pages );
		return $response;
	}

	/**
	 * Retrieve page of posts from the database
	 *
	 * @param string $post_type the post type to retrieve
	 * @param int    $page      the page number
	 *
	 * @return array<int,object>
	 */
	protected function get_page( string $post_type, int $page ): array {
		global $wpdb;

		$posts = wp_cache_get( sprintf( '%s-page-%s', $post_type, $page ), __METHOD__ );

		if ( is_array( $posts ) ) {
			return $posts;
		}

		$supported_post_statuses = Helpers::supported_post_statuses();
		$static_args             = [
			$post_type,
			( $this->posts_per_page * $page ) - $this->posts_per_page,
			$this->posts_per_page,
		];
		$dynamic_args            = array_merge( $supported_post_statuses, $static_args );
		$dynamic_placeholders    = implode( ', ', array_fill( 0, count( $supported_post_statuses ), '%s' ) );

		// Disabled for performance reasons and to allow adding dynamic placeholders into the query
		// phpcs:disable WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.PreparedSQLPlaceholders.ReplacementsWrongNumber, WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		$posts = $wpdb->get_results( $wpdb->prepare(
			"SELECT
				{$wpdb->posts}.ID,
				{$wpdb->posts}.post_title,
				{$wpdb->posts}.post_content,
				{$wpdb->posts}.post_type,
				{$wpdb->posts}.guid,
				{$wpdb->posts}.post_status
			FROM
				{$wpdb->posts}
			WHERE
				{$wpdb->posts}.post_status IN ($dynamic_placeholders)
			AND
				{$wpdb->posts}.post_type = %s
			LIMIT
				%d,%d",
			$dynamic_args
		) );
		// phpcs:enable

		wp_cache_set( sprintf( '%s-page-%s', $post_type, $page ), $posts, __METHOD__, HOUR_IN_SECONDS );

		return $posts;
	}

	/**
	 * Process a page of posts
	 *
	 * @param array<int,\WP_Post> $page       the page of posts
	 * @param bool                $is_image   whether the post is an image
	 * @param array               $media_urls list of URIs for the image
	 * @param integer             $media_id   ID of selected media
	 *
	 * @return array
	 */
	protected function process_page( array $page, bool $is_image, array $media_urls, int $media_id ): array {
		$items = [];

		// reset script execution time
		set_time_limit( absint( ini_get( 'max_execution_time' ) ) );

		foreach ( $page as $post ) {
			$processed = $this->process_post( $post, $is_image, $media_urls, $media_id );

			if ( is_null( $processed ) ) {
				continue;
			}

			$items[] = $processed;

			unset( $processed );
		}

		unset( $post );

		return $items;
	}

	/**
	 * Check whether a post should be included in the result set
	 *
	 * @param object  $post       the post object
	 * @param bool    $is_image   whether the post is an image
	 * @param array   $media_urls list of URIs for the image
	 * @param integer $media_id   ID of the selected media
	 *
	 * @return array|null
	 */
	protected function process_post( object $post, bool $is_image, array $media_urls, int $media_id ): ?array {
		$found   = null;
		$located = [];

		// check post meta if available
		if ( count( $this->post_meta_keys ) > 0 ) {
			if ( $this->process_meta( $post->ID, $media_urls, $media_id ) ) {
				$located[] = 'meta';
			}
		}

		// check content
		if ( $this->process_content( $post, $media_urls, $is_image ) ) {
			$located[] = 'content';
		}

		if ( count( $located ) > 0 ) {
			$found = [
				'id'          => $post->ID,
				'title'       => $post->post_title,
				'post_type'   => $post->post_type,
				'guid'        => $post->guid,
				'location'    => $located,
				'edit'        => get_edit_post_link( $post->ID, null ),
				'post_url'    => is_post_publicly_viewable( $post->ID ) ? get_permalink( $post->ID ) : '',
				'post_status' => $post->post_status,
			];
		}

		return $found;
	}

	/**
	 * Generates an array of media size strings to be used when searching
	 * post types for image media
	 *
	 * @param array   $media_meta Meta details of the attachment
	 * @param integer $media_id   ID of currently selected media
	 *
	 * @return array|WP_Error
	 */
	private function generate_image_media_urls( array $media_meta, int $media_id ): array | WP_Error {
		$cache_key  = md5( wp_json_encode( $media_meta ) );
		$image_urls = wp_cache_get( $cache_key, __METHOD__ );

		if ( is_array( $image_urls ) ) {
			return $image_urls;
		}

		$image_urls = [];

		if (
			! $media_meta ||
			empty( $media_meta['file'] )
		) {
			return new WP_Error(
				'attachment_meta_not_found',
				'Error finding correct attachment meta',
				[
					'status' => 404,
				]
			);
		}

		$image_urls[]    = $media_meta['file'];
		$media_file_path = explode( '/', $media_meta['file'] );
		array_pop( $media_file_path );
		$media_file_path = implode( '/', $media_file_path );
		$media_sizes     = $media_meta['sizes'] ?? [];
		$media_sizes     = is_array( $media_sizes ) ? $media_sizes : [];

		// add original image if available
		if ( isset( $media_meta['original_image'] ) ) {
			$image_urls[] = $media_file_path . '/' . $media_meta['original_image'];
		}

		foreach ( $media_sizes as $size ) {
			if ( ! isset( $size['file'] ) ) {
				continue;
			}

			$image_urls[] = $media_file_path . '/' . $size['file'];
		}

		/**
		 * Filters image urls array to be used when searching for media
		 *
		 * @param array   $image_urls A list of image URLs
		 * @param integer $media_id   Currently selected media id
		 */
		$image_urls = apply_filters( 'bigbite_content_query_media_search_image_urls', $image_urls, $media_id );

		wp_cache_set( $cache_key, $image_urls, __METHOD__ );

		return $image_urls;
	}

	/**
	 * Processes a posts post meta for the selected image
	 *
	 * @param integer $post_id    Post id
	 * @param array   $media_urls List of URIs for the media
	 * @param integer $media_id   ID of the media
	 *
	 * @return bool
	 */
	private function process_meta( int $post_id, array $media_urls, int $media_id ): bool {
		foreach ( $this->post_meta_keys as $meta_key ) {
			$meta_value = get_post_meta( $post_id, $meta_key, true );

			if ( empty( $meta_value ) ) {
				continue;
			}

			foreach ( $media_urls as $media_url ) {
				if ( str_contains( $meta_value, $media_url ) || (string) $meta_value === (string) $media_id ) {
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Processes a posts content for the selected image
	 *
	 * @param object $post       The post object
	 * @param array  $media_urls List of URIs for the media
	 * @param bool   $is_image   Whether the post is an image
	 *
	 * @return bool
	 */
	private function process_content( object $post, array $media_urls, bool $is_image ): bool {
		$rendered_post_content = do_blocks( $post->post_content );
		$featured_image        = get_the_post_thumbnail( $post->ID );

		foreach ( $media_urls as $url ) {
			$found_in_content = str_contains( $rendered_post_content, $url );

			if ( $is_image && ! str_contains( $featured_image, $url ) && ! $found_in_content ) {
				continue;
			}

			if ( ! $is_image && ! $found_in_content ) {
				continue;
			}

			return true;
		}

		return false;
	}

	/**
	 * Generates an array of strings to be used when searching
	 * post types for attachments other than images
	 *
	 * @param integer $id Attachment id
	 *
	 * @return array|WP_Error
	 */
	private function generate_other_media_urls( int $id ): array | WP_Error {
		$attachment_url = wp_get_attachment_url( $id );

		if (
			wp_http_validate_url( $attachment_url ) &&
			! empty( wp_parse_url( $attachment_url )['path'] )
		) {
			return [ wp_parse_url( $attachment_url )['path'] ];
		}

		return new WP_Error(
			'attachment_url_invalid',
			'Could not find a valid attachment URL',
			[
				'status' => 404,
			]
		);
	}
}
