<?php

namespace BigBite\Content_Query;

use WP_Error;
use WP_REST_Request;
use WP_REST_Response;

/**
 * Class responsible for registering the plugins endpoint
 */
class Rest_API_Plugins extends Rest_API_Base {

	/**
	 * Registers the rest api endpoint for this class
	 *
	 * @return void
	 */
	public function register_routes(): void {
		register_rest_route(
			$this->namespace, '/plugin/', [
				'methods'             => 'GET',
				'callback'            => [ $this, 'active_plugin_check' ],
				'permission_callback' => [ $this, 'rest_api_permissions_check' ],
				'args'                => [
					'key' => [
						'required' => true,
						'type'     => 'string',
					],
				],
			]
		);

		register_rest_route(
			$this->namespace, '/plugins/', [
				'methods'             => 'GET',
				'callback'            => [ $this, 'active_plugins_check' ],
				'permission_callback' => [ $this, 'rest_api_permissions_check' ],
			]
		);
	}

	/**
	 * Returns an array of sites where a plugin is activated or true if
	 * a plugin is network activated
	 *
	 * @param WP_REST_Request $request Request data
	 *
	 * @return WP_REST_Response
	 */
	public function active_plugin_check( WP_REST_Request $request ): WP_REST_Response {
		if ( ! function_exists( 'is_plugin_active_for_network' ) || ! function_exists( 'is_plugin_active' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$plugin_key    = sanitize_text_field( $request->get_param( 'key' ) );
		$allowed_sites = Helpers::allowed_sites();
		$active_sites  = [];

		if ( is_plugin_active_for_network( $plugin_key ) ) {
			return rest_ensure_response( true );
		}

		foreach ( $allowed_sites as $site ) {
			if ( empty( $site['id'] ) ) {
				continue;
			}

			if ( ! empty( $site['default'] ) && is_plugin_active( $plugin_key ) ) {
				$site['dashboard'] = get_dashboard_url();
				$active_sites[]    = $site;
				continue;
			}

			switch_to_blog( $site['id'] );
			if ( is_plugin_active( $plugin_key ) ) {
				$site['dashboard'] = get_dashboard_url();
				$active_sites[]    = $site;
			}
			restore_current_blog();
		}

		return rest_ensure_response( $active_sites );
	}

	/**
	 * Returns an array of sites, plugins and status data
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	public function active_plugins_check(): WP_REST_Response | WP_Error {
		if ( ! function_exists( 'is_plugin_active_for_network' ) || ! function_exists( 'is_plugin_active' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$installed_plugins       = get_plugins();
		$installed_plugins_names = array_column( $installed_plugins, 'Name' );
		$installed_plugins_keys  = array_keys( $installed_plugins );
		$allowed_sites           = Helpers::allowed_sites();
		$matrix                  = [];

		if ( 0 === count( $installed_plugins ) ) {
			return new WP_Error(
				'no_plugins_installed',
				'No installed plugins found',
				[
					'status' => 404,
				]
			);
		}

		if ( count( $installed_plugins_names ) !== count( $installed_plugins_keys ) ) {
			return new WP_Error(
				'plugins_key_name_total_mismatch',
				'There was a plugin name/key total mismatch',
				[
					'status' => 500,
				]
			);
		}

		foreach ( $allowed_sites as $site ) {
			if ( empty( $site['id'] ) ) {
				continue;
			}

			switch_to_blog( $site['id'] );

			foreach ( $installed_plugins_keys as $key => $plugin ) {
				if (
					! is_plugin_active_for_network( $plugin ) &&
					! is_plugin_active( $plugin )
				) {
					$matrix[ $key ][] = false;
					continue;
				}

				$matrix[ $key ][] = true;
			}

			restore_current_blog();
		}

		return rest_ensure_response( [
			'plugins' => $installed_plugins_names,
			'sites'   => $allowed_sites,
			'matrix'  => $matrix,
		] );
	}
}
