<?php

namespace BigBite\Content_Query;

use WP_Query;
use WP_REST_Request;
use WP_REST_Response;

/**
 * Class responsible for registering the post endpoint
 */
class Rest_API_Posts extends Rest_API_Base {

	/**
	 * Class constructor
	 */
	public function __construct() {
		parent::__construct();

		/**
		 * Filters post explorers posts per page query parameter
		 *
		 * @param int Posts per page
		 */
		$this->posts_per_page = apply_filters( 'bigbite_content_query_post_explorer_posts_per_page', 5 );
	}

	/**
	 * Registers the rest api endpoint for this class
	 *
	 * @return void
	 */
	public function register_routes(): void {
		register_rest_route(
			$this->namespace, '/posts/', [
				'methods'             => 'GET',
				'callback'            => [ $this, 'get_posts_with_blocks' ],
				'permission_callback' => [ $this, 'rest_api_permissions_check' ],
				'args'                => [
					'page'       => [
						'default'           => 1,
						'validate_callback' => [ $this, 'page_validate_callback' ],
						'sanitize_callback' => 'absint',
					],
					'post_types' => [
						'default'           => $this->supported_post_types,
						'validate_callback' => [ $this, 'post_types_validate_callback' ],
						'sanitize_callback' => [ $this, 'post_types_sanitize_callback' ],
					],
					's'          => [
						'validate_callback' => [ $this, 'search_input_validate_callback' ],
						'sanitize_callback' => 'sanitize_text_field',
					],
				],
			]
		);
	}

	/**
	 * Validation callback for the 'page' request parameter in get_posts_with_blocks
	 *
	 * @param mixed           $param   Current request parameter value
	 * @param WP_REST_Request $request Request data
	 *
	 * @return bool
	 */
	public function page_validate_callback( mixed $param, WP_REST_Request $request ): bool {
		if ( ! is_int( $param ) && ! ctype_digit( $param ) ) {
			return false;
		}

		// check the page requested isn't greater than the total possible pages
		$requested_post_types = $request->get_param( 'post_types' );
		$requested_post_types = is_array( $requested_post_types ) ? $requested_post_types : [];
		$post_types           = array_intersect( $this->supported_post_types, $requested_post_types );
		$post_types_total     = 0;

		foreach ( $post_types as $post_type ) {
			$post_types_total += Helpers::post_type_post_status_total_count( $post_type );
		}

		if (
			$param <= ceil( $post_types_total / $this->posts_per_page ) ||
			0 === $post_types_total
		) {
			return true;
		}

		return false;
	}

	/**
	 * Validation callback for the 'post_types' request parameter in get_posts_with_blocks
	 *
	 * @param mixed $param Current request parameter value
	 *
	 * @return bool
	 */
	public function post_types_validate_callback( mixed $param ): bool {
		return is_array( $param );
	}

	/**
	 * Sanitize callback for the 'post_types' request parameter in get_posts_with_blocks
	 *
	 * @param array $param Current request parameter value
	 *
	 * @return array
	 */
	public function post_types_sanitize_callback( array $param ): array {
		return array_intersect( $this->supported_post_types, $param );
	}

	/**
	 * Validation callback for the 's' request parameter in get_posts_with_blocks
	 *
	 * @param mixed $param Current request parameter value
	 *
	 * @return bool
	 */
	public function search_input_validate_callback( mixed $param ): bool {
		return is_string( $param );
	}

	/**
	 * Returns the post meta from the post id
	 *
	 * @param int $post_id Post id
	 *
	 * @return array|null
	 */
	private function get_flattened_posts_meta( int $post_id ): ?array {
		$post_meta           = get_post_meta( $post_id );
		$flattened_post_meta = [];

		if ( ! $post_meta ) {
			return $flattened_post_meta;
		}

		foreach ( $post_meta as $key => $value ) {
			if ( ! isset( $value[0] ) ) {
				continue;
			}

			$flattened_post_meta[ $key ] = $value[0];
		}

		return $flattened_post_meta;
	}

	/**
	 * Returns posts and their corresponding parsed blocks based on request parameters
	 *
	 * @param WP_REST_Request $request Request data
	 *
	 * @return WP_REST_Response
	 */
	public function get_posts_with_blocks( WP_REST_Request $request ): WP_REST_Response {
		$posts      = [];
		$s          = $request->get_param( 's' );
		$page       = $request->get_param( 'page' );
		$post_types = $request->get_param( 'post_types' );

		$args = [
			'post_type'      => $post_types,
			'paged'          => $page,
			'posts_per_page' => $this->posts_per_page,
			's'              => $s,
			'post_status'    => Helpers::supported_post_statuses(),
		];

		$post_query  = new WP_Query( $args );
		$total_pages = $post_query->max_num_pages;

		if ( $post_query->have_posts() ) {
			while ( $post_query->have_posts() ) {
				$post_query->the_post();
				$posts[] = [
					'id'          => get_the_ID(),
					'title'       => get_the_title(),
					'url'         => is_post_publicly_viewable() ? get_permalink() : '',
					'type'        => get_post_type(),
					'blocks'      => parse_blocks( ! post_password_required() ? get_the_content() : $post_query?->post?->post_content ),
					'edit'        => get_edit_post_link( get_the_ID(), null ),
					'date'        => get_the_date(),
					'author'      => get_the_author(),
					'post_status' => get_post_status(),
					'post_meta'   => $this->get_flattened_posts_meta( get_the_ID() ),
				];
			}

			wp_reset_postdata();
		}

		$response = rest_ensure_response( $posts );
		$response->header( 'X-WP-TotalPages', $total_pages );
		return $response;
	}
}
