<?php

namespace BigBite\Content_Query;

use WP_Error;
use WP_REST_Request;
use WP_REST_Response;

/**
 * Class responsible for registering the themes endpoint
 */
class Rest_API_Themes extends Rest_API_Base {

	/**
	 * Registers the rest api endpoint for this class
	 *
	 * @return void
	 */
	public function register_routes(): void {
		register_rest_route(
			$this->namespace, '/theme/', [
				'methods'             => 'GET',
				'callback'            => [ $this, 'active_theme_check' ],
				'permission_callback' => [ $this, 'rest_api_permissions_check' ],
				'args'                => [
					'key' => [
						'required' => true,
						'type'     => 'string',
					],
				],
			]
		);

		register_rest_route(
			$this->namespace, '/themes/', [
				'methods'             => 'GET',
				'callback'            => [ $this, 'active_themes_check' ],
				'permission_callback' => [ $this, 'rest_api_permissions_check' ],
			]
		);
	}

	/**
	 * Returns an array of sites where a theme is activated on a multisite network
	 *
	 * @param WP_REST_Request $request Request data
	 *
	 * @return WP_REST_Response
	 */
	public function active_theme_check( WP_REST_Request $request ): WP_REST_Response {
		$theme_key     = sanitize_text_field( $request->get_param( 'key' ) );
		$allowed_sites = Helpers::allowed_sites();
		$active_sites  = [];

		foreach ( $allowed_sites as $site ) {
			if ( empty( $site['id'] ) ) {
				continue;
			}

			if ( ! empty( $site['default'] ) && wp_get_theme()->get( 'Name' ) === $theme_key ) {
				$site['dashboard'] = get_dashboard_url();
				$active_sites[]    = $site;
				continue;
			}

			switch_to_blog( $site['id'] );
			if ( wp_get_theme()->get( 'Name' ) === $theme_key ) {
				$site['dashboard'] = get_dashboard_url();
				$active_sites[]    = $site;
			}
			restore_current_blog();
		}

		return rest_ensure_response( $active_sites );
	}

	/**
	 * Returns an array of sites, themes and status data
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	public function active_themes_check(): WP_REST_Response | WP_Error {
		$installed_themes = array_column( Helpers::get_themes(), 'name' );
		$allowed_sites    = Helpers::allowed_sites();
		$matrix           = [];

		if ( 0 === count( $installed_themes ) ) {
			return new WP_Error(
				'no_themes_installed',
				'No installed themes found',
				[
					'status' => 404,
				]
			);
		}

		foreach ( $allowed_sites as $site ) {
			if ( empty( $site['id'] ) ) {
				continue;
			}

			switch_to_blog( $site['id'] );

			$active_theme = wp_get_theme()?->get( 'Name' );

			foreach ( $installed_themes as $key => $theme ) {
				if ( $theme !== $active_theme ) {
					$matrix[ $key ][] = false;
					continue;
				}

				$matrix[ $key ][] = true;
			}

			restore_current_blog();
		}

		return rest_ensure_response( [
			'themes' => $installed_themes,
			'sites'  => $allowed_sites,
			'matrix' => $matrix,
		] );
	}
}
